-- CREACION DE TABLAS

-- APLICACION
CREATE TABLE APLICACION (
  id_aplicacion VARCHAR2(40) NOT NULL,
  version VARCHAR2(20) NOT NULL,
  fecha_subida DATE NOT NULL,
  url_demostracion VARCHAR2(200),
  resolucion_pantalla VARCHAR2(20) NOT NULL,
  activa CHAR(2) NOT NULL, 
  CONSTRAINT PK_APLICACION PRIMARY KEY (id_aplicacion)
);

-- PAIS
CREATE TABLE PAIS (
  id_pais CHAR(2) NOT NULL,
  CONSTRAINT PK_PAIS PRIMARY KEY (id_pais)
);

-- PRECIO
CREATE TABLE PRECIO (
  aplicacion VARCHAR2(40) NOT NULL,
  pais CHAR(2) NOT NULL,
  precio NUMBER,
  CONSTRAINT PK_PRECIO PRIMARY KEY (aplicacion, pais),
  CONSTRAINT FK_PRECIO_APLICACION FOREIGN KEY (aplicacion) REFERENCES APLICACION(id_aplicacion),
  CONSTRAINT FK_PRECIO_PAIS FOREIGN KEY (pais) REFERENCES PAIS(id_pais)
);

-- DESCRIPCION
CREATE TABLE DESCRIPCION (
  idioma VARCHAR2(20) NOT NULL,
  aplicacion VARCHAR2(40) NOT NULL,
  descripcion VARCHAR2(300) NOT NULL,
  CONSTRAINT PK_DESCRIPCION PRIMARY KEY (idioma, aplicacion),
  CONSTRAINT FK_DESCRIPCION_APLICACION FOREIGN KEY (aplicacion) REFERENCES APLICACION(id_aplicacion)
);

-- SISTEMA_OPERATIVO
CREATE TABLE SIST_OP (
  id_so VARCHAR2(20) NOT NULL,
  CONSTRAINT PK_SIST_OP PRIMARY KEY (id_so)
);

-- DISPONIBLE
CREATE TABLE DISPONIBLE (
  aplicacion VARCHAR2(40) NOT NULL,
  sistema_operativo VARCHAR2(20) NOT NULL,
  tamanyo NUMBER NOT NULL,
  enlace_instalacion VARCHAR2(100) NOT NULL, 
  CONSTRAINT PK_DISPONIBLE PRIMARY KEY (aplicacion, sistema_operativo),
  CONSTRAINT FK_DISPONIBLE_APLICACION FOREIGN KEY (aplicacion) REFERENCES APLICACION(id_aplicacion),
  CONSTRAINT FK_DISPONIBLE_SIST_OP FOREIGN KEY (sistema_operativo) REFERENCES SIST_OP(id_so)
);

-- USUARIO
CREATE TABLE USUARIO (
  num_movil NUMBER NOT NULL,
  operador VARCHAR2(20) NOT NULL,
  correo_electronico VARCHAR2(40) NOT NULL,
  pais CHAR(2) NOT NULL,
  CONSTRAINT PK_USUARIO PRIMARY KEY (num_movil),
  CONSTRAINT FK_USUARIO_PAIS FOREIGN KEY (pais) REFERENCES PAIS(id_pais)
);

-- EMPRESA
CREATE TABLE EMPRESA (
  id_empresa NUMBER NOT NULL,
  nombre_empresa VARCHAR2(40) NOT NULL,
  nombre_representante VARCHAR2(40) NOT NULL,
  pais CHAR(2) NOT NULL,
  direccion_central VARCHAR2(70) NOT NULL,
  telefono NUMBER NOT NULL,
  CONSTRAINT PK_EMPRESA PRIMARY KEY (id_empresa),
  CONSTRAINT FK_EMPRESA_PAIS FOREIGN KEY (pais) REFERENCES PAIS(id_pais)
);

-- DESARROLLADOR
CREATE TABLE DESARROLLADOR (
  d_num_movil NUMBER NOT NULL,
  empresa NUMBER,
  CONSTRAINT PK_DESARROLLADOR PRIMARY KEY (d_num_movil),
  CONSTRAINT FK_DESARROLLADOR_USUARIO FOREIGN KEY (d_num_movil) REFERENCES USUARIO(num_movil),
  CONSTRAINT FK_DESARROLLADOR_EMPRESA FOREIGN KEY (empresa) REFERENCES EMPRESA(id_empresa)
);

-- DESARROLLA
CREATE TABLE DESARROLLA (
  desarrollador NUMBER NOT NULL,
  aplicacion VARCHAR2(40) NOT NULL,
  CONSTRAINT PK_DESARROLLA PRIMARY KEY (desarrollador, aplicacion),
  CONSTRAINT FK_DESARROLLA_DESARROLLADOR FOREIGN KEY (desarrollador) REFERENCES DESARROLLADOR(d_num_movil),
  CONSTRAINT FK_DESARROLLA_APLICACION FOREIGN KEY (aplicacion) REFERENCES APLICACION(id_aplicacion)
);

-- DISPOSITIVO
CREATE TABLE DISPOSITIVO (
  codigo_imei NUMBER(15) NOT NULL,
  sistema_operativo VARCHAR2(20) NOT NULL,
  modelo VARCHAR2(30) NOT NULL,
  resolucion VARCHAR2(20) NOT NULL,
  usuario NUMBER NOT NULL,
  CONSTRAINT PK_DISPOSITIVO PRIMARY KEY (codigo_imei),
  CONSTRAINT FK_DISPOSITIVO_USUARIO FOREIGN KEY (usuario) REFERENCES USUARIO(num_movil)
);

-- DESCARGA
CREATE TABLE DESCARGA (
  aplicacion VARCHAR2(40) NOT NULL,
  dispositivo NUMBER(15) NOT NULL,
  usuario NUMBER NOT NULL,
  pais CHAR(2) NOT NULL,
  fecha DATE NOT NULL,
  forma_de_pago VARCHAR2(22) NOT NULL,
  precio NUMBER NOT NULL,
  operador VARCHAR2(20) NOT NULL,
  CONSTRAINT PK_DESCARGA PRIMARY KEY (aplicacion, dispositivo),
  CONSTRAINT FK_DESCARGA_APLICACION FOREIGN KEY (aplicacion) REFERENCES APLICACION(id_aplicacion),
  CONSTRAINT FK_DESCARGA_DISPOSITIVO FOREIGN KEY (dispositivo) REFERENCES DISPOSITIVO(codigo_imei),
  CONSTRAINT FK_DESCARGA_USUARIO FOREIGN KEY (usuario) REFERENCES USUARIO(num_movil), 
  CONSTRAINT FK_DESCARGA_PAIS FOREIGN KEY (pais) REFERENCES PAIS(id_pais), 
  CONSTRAINT CK_DESCARGA CHECK (forma_de_pago IN('paypal', 'tarjeta de credito', 'transferencia bancaria'))
);
 
-- Modulo estadístico 1
CREATE TABLE EST1_DESCARGAS (
  num_descargas NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST1_DESCARGAS PRIMARY KEY (num_descargas)
);

-- Modulo estadístico 2
CREATE TABLE EST2_RECAUDACION (
  recaudacion NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST2_RECAUDACION PRIMARY KEY (recaudacion)
);

-- Modulo estadístico 3
CREATE TABLE EST3_MEDIA_DESCARGAS (
  anyo_est3 NUMBER(4) NOT NULL,
  media_descargas NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST3_Media_DESCARGAS PRIMARY KEY (anyo_est3)
);

-- Modulo estadístico 4
CREATE TABLE EST4_DESARROLLADOR_MAX (
  anyo_est4 NUMBER(4) NOT NULL,
  desarrollador NUMBER DEFAULT 0 NOT NULL,
  num_descargas NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST4_DESARROLLADOR_MAX PRIMARY KEY (anyo_est4)
);

-- Modulo estadístico 5
CREATE TABLE EST5_APLICACION_MAX_REC (
  anyo_est5 NUMBER(4) NOT NULL,
  aplicacion VARCHAR2(40) NOT NULL,
  desarrollador NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST5_APLICACION_MAX_REC PRIMARY KEY (anyo_est5)
);

-- Modulo estadístico 6
CREATE TABLE EST6_USUARIOS_DESCARGAS (
  anyo_est6 NUMBER(4) NOT NULL,
  pais_est6 CHAR(2) NOT NULL,
  num_usuarios NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST6_USUARIOS_DESCARGAS PRIMARY KEY (anyo_est6, pais_est6)
);

-- Modulo estadístico 7
CREATE TABLE EST7_INGRESOS_PAIS (
  anyo_est7 NUMBER(4) NOT NULL,
  pais_est7 CHAR(2) NOT NULL,
  ingresos NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST7_INGRESOS_PAIS PRIMARY KEY (anyo_est7, pais_est7)
);

-- Modulo estadístico 8
CREATE TABLE EST8_APLICACIONES_PAIS (
  anyo_est8 NUMBER(4) NOT NULL,
  pais_est8 CHAR(2) NOT NULL,
  aplicaciones_descargadas NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_EST8_APLICACIONES_PAIS PRIMARY KEY (anyo_est8, pais_est8)
);

-- LOG
CREATE TABLE LOGS (
  codigo NUMBER NOT NULL,
  fecha_hora VARCHAR2(25) NOT NULL,
  nombre_procedimiento VARCHAR2(40) NOT NULL,
  param_entrada VARCHAR2(400) NOT NULL,
  param_salida VARCHAR2(400) NOT NULL, 
  CONSTRAINT PK_LOGS PRIMARY KEY (codigo)
);

CREATE SEQUENCE SQ_LOGS INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TR_BI_LOGS
  BEFORE INSERT ON LOGS
  FOR EACH ROW
BEGIN
  SELECT SQ_LOGS.NEXTVAL INTO :NEW.codigo
  FROM DUAL;
END TR_BI_LOGS;
/

