create or replace PROCEDURE PR_EST8 (
   p_anyo IN EST8_APLICACIONES_PAIS.anyo_est8%TYPE,
   p_pais IN EST8_APLICACIONES_PAIS.pais_est8%TYPE
)
IS
/******************************************************************************
   Nombre:              PR_EST8

   Propsito:           Aade una entrada a la tabla estadstica EST8_APLICACIONES_PAIS. 
                        Esta informacin corresponde al nmero de aplicaciones
			diferentes descargadas al menos una vez en el pas y 
			ao indicados.
                   
   Parmetros entrada:  p_anyo, p_pais          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
RSP  VARCHAR2 (500);
existe INTEGER;
p_aplicaciones_descargadas NUMBER;
err_msg VARCHAR2 (200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;

BEGIN

   param_entrada:='p_anyo: '||to_char(p_anyo)||', p_pais: '||p_pais;

  --verificamos que los campos obligatorios no sean nulos
   IF (p_anyo IS NULL) THEN 
     campo_nulo:='anyo';
     RAISE E_NULO;
   ELSIF (p_pais IS NULL) THEN 
     campo_nulo:='pais';
     RAISE E_NULO;  
   END IF;

  -- calculamos los ingresos generados en el pas y ao indicados
   SELECT COUNT(DISTINCT aplicacion)
   INTO p_aplicaciones_descargadas
   FROM DESCARGA d
   WHERE EXTRACT(YEAR FROM d.fecha)=p_anyo AND pais=p_pais;

  -- calculamos si existe la entrada en la tabla EST8_APLICACIONES_PAIS correspondiente al ao indicado
   SELECT COUNT (*)
   INTO existe
   FROM EST8_APLICACIONES_PAIS
   WHERE anyo_est8=p_anyo AND pais_est8=p_pais;   
  --Si no existe la entrada, la aadimos 
   IF (existe=0) THEN 
     INSERT INTO EST8_APLICACIONES_PAIS (anyo_est8, pais_est8, aplicaciones_descargadas) 
     VALUES (p_anyo, p_pais, p_aplicaciones_descargadas);     
  --Si existe la actualizamos
   ELSE
     UPDATE EST8_APLICACIONES_PAIS 
     SET aplicaciones_descargadas=p_aplicaciones_descargadas
     WHERE anyo_est8=p_anyo AND pais_est8=p_pais;
   END IF;  
 
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST8', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR EST8 el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST8', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST8: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST8', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
              
END PR_EST8;