create or replace PROCEDURE PR_ALTA_LOGS (
   p_fecha_hora IN LOGS.fecha_hora%TYPE,
   p_nombre_procedimiento IN LOGS.nombre_procedimiento%TYPE,
   p_param_entrada IN LOGS.param_entrada%TYPE,
   p_param_salida IN LOGS.param_salida%TYPE
)
IS
/******************************************************************************
   Nombre:              PR_ALTA_LOGS

   Propsito:           aade una entrada a la tabla LOGS
                   
   Parmetros entrada:  p_fecha_hora, p_nombre_procedimiento, p_param_entrada, p_param_salida            

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
RSP VARCHAR2 (500);
err_msg VARCHAR2 (200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;

  -- marcamos este bloque como transaccin autnoma para que, aunque se produzca un error en 
  --la transaccin que lo ejecute y se realice un rollback posterior, quede un registro en el log de ello
PRAGMA AUTONOMOUS_TRANSACTION;

BEGIN

   param_entrada:='p_fecha_hora: '||p_fecha_hora||', p_nombre_procedimiento: '||p_nombre_procedimiento||', p_param_entrada: '||p_param_entrada||', p_param_salida: '||p_param_salida;

  -- verificamos que los campos obligatorios no sean nulos
   IF (p_fecha_hora IS NULL) THEN 
     campo_nulo:='fecha_hora';
     RAISE E_NULO;
   ELSIF (p_nombre_procedimiento IS NULL) THEN 
     campo_nulo:='nombre_procedimiento';     
     RAISE E_NULO;
   ELSIF (p_param_entrada IS NULL) THEN 
     campo_nulo:='param_entrada';
     RAISE E_NULO;     
   ELSIF (p_param_salida IS NULL) THEN 
     campo_nulo:='param_salida';
     RAISE E_NULO;   
   END IF;
 
  -- insertamos la nueva entrada en la tabla logs
   INSERT INTO LOGS (fecha_hora, nombre_procedimiento, param_entrada, param_salida)
   VALUES (p_fecha_hora, p_nombre_procedimiento, p_param_entrada, p_param_salida);
   
  -- finaliza con xito
   RSP:='OK';


   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR LOG: el campo '||campo_nulo||' no puede ser nulo'; 
        INSERT INTO LOGS (fecha_hora, nombre_procedimiento, param_entrada, param_salida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_LOGS', p_param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo        
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR LOG: '||err_msg;
        INSERT INTO LOGS (fecha_hora, nombre_procedimiento, param_entrada, param_salida)
        VALUES (to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_LOGS', p_param_entrada,'RSP: '||RSP);
        ROLLBACK;
              
END PR_ALTA_LOGS;