create or replace PROCEDURE PR_EST2 
IS
/******************************************************************************
   Nombre:              PR_EST2

   Propsito:           Modifica el valor de la entrada de la tabla EST2_RECAUDACION. 
                        Esta informacin corresponde al nmero total de euros generados en 
			descargas hasta el momento. 
                   
   Parmetros entrada:  sin parmetros            

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
RSP VARCHAR2 (500);
existe INTEGER;
p_recaudacion NUMBER;
err_msg VARCHAR2 (200);

BEGIN

   param_entrada:='sin parametros';

  -- calculamos el nuevo nmero total de euros generados en descargas hasta el momento
   SELECT SUM (precio)
   INTO p_recaudacion
   FROM DESCARGA;

  -- calculamos si existe una entrada en la tabla EST2_RECAUDACION
   SELECT COUNT (*)
   INTO existe
   FROM EST2_RECAUDACION;   
  --Si no existe la entrada, la aadimos 
   IF (existe=0) THEN 
     INSERT INTO EST2_RECAUDACION (recaudacion) VALUES (p_recaudacion);     
  --Si existe la actualizamos
   ELSE
     UPDATE EST2_RECAUDACION 
     SET recaudacion=p_recaudacion;
   END IF;  

  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST2', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finaliza con xito       
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR LOG: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST1', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
              
END PR_EST2;