create or replace PROCEDURE PR_EST3 (
   p_anyo IN EST3_MEDIA_DESCARGAS.anyo_est3%TYPE
)
IS
/******************************************************************************
   Nombre:              PR_EST3

   Propsito:           Aade una entrada a la tabla estadstica EST3_MEDIA_DESCARGAS. 
                        Esta informacin corresponde al nmero medio de aplicaciones 
			descargadas en un ao concreto. 
                   
   Parmetros entrada:  p_anyo            

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
RSP VARCHAR2 (500);
p_media_descargas NUMBER;
descargas NUMBER;
usuarios NUMBER;
existe INTEGER;
err_msg VARCHAR2 (200);
E_NULO EXCEPTION;

BEGIN

   param_entrada:='p_anyo: '||to_char(p_anyo);

  --verificamos que el campo obligatorio no sea nulo
   IF (p_anyo IS NULL) THEN 
     RAISE E_NULO;    
   END IF;

  -- calculamos la media de descargas realizadas en el ao indicado
   SELECT COUNT (*)
   INTO descargas
   FROM DESCARGA
   WHERE EXTRACT(YEAR FROM fecha)=p_anyo;
   SELECT COUNT (*)
   INTO usuarios
   FROM USUARIO;
   p_media_descargas:=descargas/usuarios;

  -- calculamos si existe la entrada en la tabla EST3_MEDIA_DESCARGAS correspondiente al ao indicado
   SELECT COUNT (*)
   INTO existe
   FROM EST3_MEDIA_DESCARGAS
   WHERE anyo_est3=p_anyo;   
  --Si no existe la entrada, la aadimos 
   IF (existe=0) THEN 
     INSERT INTO EST3_MEDIA_DESCARGAS (anyo_est3, media_descargas) VALUES (p_anyo, p_media_descargas);     
  --Si existe la actualizamos
   ELSE
     UPDATE EST3_MEDIA_DESCARGAS 
     SET media_descargas=p_media_descargas
     WHERE anyo_est3=p_anyo;
   END IF;  
 
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST3', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finaliza con xito porque el valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR EST3 el campo anyo no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST3', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo         
   WHEN OTHERS THEN
        err_msg:=SUBSTR(SQLERRM, 1, 200);
        RSP:='ERROR EST3: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST3', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
              
END PR_EST3;