create or replace PROCEDURE PR_EST4 (
   p_anyo IN EST4_DESARROLLADOR_MAX.anyo_est4%TYPE
)
IS
/******************************************************************************
   Nombre:              PR_EST4

   Propsito:           Aade una entrada a la tabla estadstica EST4_DESARROLLADOR_MAX. 
                        Esta informacin corresponde al desarrollador con el mximo nmero 
			de descargas en sus aplicaciones y el nmero de descargas que sto
			representa en un ao concreto
                   
   Parmetros entrada:  p_anyo            

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
RSP VARCHAR2 (500);
existe INTEGER;
p_media_descargas NUMBER;
p_desarrollador NUMBER;
p_num_descargas NUMBER;
num INTEGER;
err_msg VARCHAR2 (200);
E_NULO EXCEPTION;

BEGIN

   param_entrada:='p_anyo: '||to_char(p_anyo);

  --verificamos que el campo obligatorio no sea nulo
   IF (p_anyo IS NULL) THEN 
     RAISE E_NULO;    
   END IF;

  -- calculamos el desarrollador con el mximo nmero de descargas en el ao indicado
  --y el nmero de descargas que esto representa
   SELECT desarrollador,  num 
   INTO p_desarrollador, p_num_descargas
   FROM (SELECT d.desarrollador AS desarrollador, COUNT(des.dispositivo) AS num FROM DESARROLLA d JOIN DESCARGA des ON d.aplicacion=des.aplicacion 
        WHERE EXTRACT(YEAR FROM des.fecha)=p_anyo GROUP BY d.desarrollador ORDER BY num DESC)
   WHERE rownum=1;
   
  -- calculamos si existe la entrada en la tabla EST4_DESARROLLADOR_MAX correspondiente al ao indicado
   SELECT COUNT (*)
   INTO existe
   FROM EST4_DESARROLLADOR_MAX
   WHERE anyo_est4=p_anyo;   
  --Si no existe la entrada, la aadimos 
   IF (existe=0) THEN 
     INSERT INTO EST4_DESARROLLADOR_MAX (anyo_est4, desarrollador, num_descargas) 
     VALUES (p_anyo, p_desarrollador, p_num_descargas);     
  --Si existe la actualizamos
   ELSE
     UPDATE EST4_DESARROLLADOR_MAX 
     SET desarrollador=p_desarrollador,
         num_descargas=p_num_descargas
     WHERE anyo_est4=p_anyo;
   END IF;  
 
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST4', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finaliza con xito porque el valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR EST4 el campo anyo no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST4', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST4: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST4', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
              
END PR_EST4;