create or replace PROCEDURE PR_EST5 (
   p_anyo IN EST5_APLICACION_MAX_REC.anyo_est5%TYPE
)
IS
/******************************************************************************
   Nombre:              PR_EST5

   Propsito:           Aade una entrada a la tabla estadstica EST5_APLICACION_MAX_REC. 
                        Esta informacin corresponde a la aplicacin que ms dinero 
			ha recaudado en descargas en un ao determinado y su 
			correspondiente desarrollador.
                   
   Parmetros entrada:  p_anyo            

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
RSP VARCHAR2 (500);
existe INTEGER;
p_aplicacion NUMBER;
p_desarrollador NUMBER;
suma NUMBER;
err_msg VARCHAR2 (200);
E_NULO EXCEPTION;

BEGIN
   param_entrada:='p_anyo: '||to_char(p_anyo);

  --verificamos que el campo obligatorio no sea nulo
   IF (p_anyo IS NULL) THEN 
     RAISE E_NULO;    
   END IF;

  -- hallamos la aplicacin que ms dinero ha recaudado en el ao indicado
   SELECT aplicacion, s
   INTO p_aplicacion, suma
   FROM (SELECT aplicacion, SUM(precio) AS s FROM DESCARGA WHERE EXTRACT(YEAR FROM fecha)=p_anyo GROUP BY aplicacion ORDER BY s DESC)
   WHERE rownum=1;
  -- hallamos el desarrollador de la aplicacin
   SELECT desarrollador
   INTO p_desarrollador
   FROM DESARROLLA
   WHERE aplicacion=p_aplicacion;

  -- calculamos si existe la entrada en la tabla EST5_APLICACION_MAX_REC correspondiente al ao indicado
   SELECT COUNT (*)
   INTO existe
   FROM EST5_APLICACION_MAX_REC
   WHERE anyo_est5=p_anyo;   
  --Si no existe la entrada, la aadimos 
   IF (existe=0) THEN 
     INSERT INTO EST5_APLICACION_MAX_REC (anyo_est5, aplicacion, desarrollador) 
     VALUES (p_anyo, p_aplicacion, p_desarrollador);     
  --Si existe la actualizamos
   ELSE
     UPDATE EST5_APLICACION_MAX_REC 
     SET aplicacion=p_aplicacion,
         desarrollador=p_desarrollador
     WHERE anyo_est5=p_anyo;
   END IF;  
 
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST5', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finaliza con xito porque el valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR EST5 el campo anyo no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST5', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST5: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST5', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_EST5;
