create or replace PROCEDURE PR_EST6 (
   p_anyo IN EST6_USUARIOS_DESCARGAS.anyo_est6%TYPE,
   p_pais IN EST6_USUARIOS_DESCARGAS.pais_est6%TYPE
)
IS
/******************************************************************************
   Nombre:              PR_EST6

   Propsito:           Aade una entrada a la tabla estadstica EST6_USUARIOS_DESCARGAS. 
                        Esta informacin corresponde al nmero de usuarios distintos
			que han hecho al menos una descarga en un pas y ao determinados.
                   
   Parmetros entrada:  p_anyo, p_pais          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
RSP VARCHAR2 (500);
existe INTEGER;
p_num_usuarios NUMBER;
err_msg VARCHAR2 (200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;

BEGIN

   param_entrada:='p_anyo: '||to_char(p_anyo)||', p_pais: '||p_pais;

  --verificamos que los campos obligatorios no sean nulos
   IF (p_anyo IS NULL) THEN 
     campo_nulo:='anyo';
     RAISE E_NULO;
   ELSIF (p_pais IS NULL) THEN 
     campo_nulo:='pais';
     RAISE E_NULO;  
   END IF;

  -- calculamos el nmero de usuarios que han realizado al menos una descarga en el pas y ao indicados
   SELECT COUNT(DISTINCT usuario)
   INTO p_num_usuarios
   FROM DESCARGA d
   WHERE EXTRACT(YEAR FROM d.fecha)=p_anyo AND pais=p_pais;

  -- calculamos si existe la entrada en la tabla EST6_USUARIOS_DESCARGAS correspondiente al ao indicado
   SELECT COUNT (*)
   INTO existe
   FROM EST6_USUARIOS_DESCARGAS
   WHERE anyo_est6=p_anyo AND pais_est6=p_pais;   
  --Si no existe la entrada, la aadimos 
   IF (existe=0) THEN 
     INSERT INTO EST6_USUARIOS_DESCARGAS (anyo_est6, pais_est6, num_usuarios) 
     VALUES (p_anyo, p_pais, p_num_usuarios);     
  --Si existe la actualizamos
   ELSE
     UPDATE EST6_USUARIOS_DESCARGAS 
     SET num_usuarios=p_num_usuarios
     WHERE anyo_est6=p_anyo AND pais_est6=p_pais;
   END IF;  
 
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST6', param_entrada, 'RSP: '||RSP);

   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR EST6 el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST6', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo         
   WHEN OTHERS THEN
        err_msg :=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR EST6: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_EST6', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
              
END PR_EST6;