create or replace PROCEDURE PR_BAJA_APLICACION (
   p_id_aplicacion IN APLICACION.id_aplicacion%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_BAJA_APLICACION

   Propsito:           Elimina una aplicacin del sistema. Tambin elimina los precios,
			descripciones, descargas y entradas de las tablas DISPONIBLE y 
			DESARROLLA asociados a la aplicacin.
                   
   Parmetros entrada:  p_id_aplicacion  

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs
   param_entrada:='p_id_aplicacion: '||p_id_aplicacion;
     
  --verificamos que el campo obligatorio no sea nulo
   IF (p_id_aplicacion IS NULL) THEN 
     RAISE E_NULO;
   END IF; 

  --verificamos si existe la aplicacin
   SELECT COUNT (*)
   INTO existe
   FROM APLICACION
   WHERE id_aplicacion=p_id_aplicacion;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

   -- eliminamos los precios relacionados con la aplicacin
   DELETE FROM PRECIO
   WHERE aplicacion=p_id_aplicacion;

   -- eliminamos las descripciones relacionadas con la aplicacin
   DELETE FROM DESCRIPCION
   WHERE aplicacion=p_id_aplicacion;

   -- eliminamos las entradas de la tabla DISPONIBLE relacionadas con la aplicacin
   DELETE FROM DISPONIBLE
   WHERE aplicacion=p_id_aplicacion;

   -- eliminamos las entradas de la tabla DESARROLLA relacionadas con la aplicacin
   DELETE FROM DESARROLLA
   WHERE aplicacion=p_id_aplicacion;

  -- eliminamos las descargas asociadas
   DELETE FROM DESCARGA
   WHERE aplicacion=p_id_aplicacion;
   
  -- eliminamos la aplicacin
   DELETE FROM APLICACION
   WHERE id_aplicacion=p_id_aplicacion;


  -- finaliza con xito  
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_APLICACION', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque el valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo id_aplicacion no puede ser nulo';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_APLICACION', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe una aplicacin con el mismo id_aplicacion     
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: la aplicacin no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_APLICACION', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo     
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_APLICACION', param_entrada, 'RSP: '||RSP);  
        ROLLBACK;    
              
END PR_BAJA_APLICACION;
