create or replace PROCEDURE PR_MODIFICACION_APLICACION (
   p_id_aplicacion IN APLICACION.id_aplicacion%TYPE,
   p_version IN APLICACION.version%TYPE,
   p_fecha_subida IN APLICACION.fecha_subida%TYPE,
   p_url_demostracion IN APLICACION.url_demostracion%TYPE,
   p_resolucion_pantalla IN APLICACION.resolucion_pantalla%TYPE,
   p_activa IN APLICACION.activa%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_MODIFICACION_APLICACION
   Propsito:           Modifica los datos de una entrada de la tabla APLICACION
                   
   Parmetros entrada:  p_id_aplicacion, p_version, p_fecha_subida, p_url_demostracion, 
			p_resolucion_pantalla, p_activa   

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:='p_id_aplicacion: '||p_id_aplicacion||', p_version: '||p_version||', p_fecha_subida: '||to_char(p_fecha_subida)||', p_url_demostracion: '||p_url_demostracion||', p_resolucion_pantalla: '||p_resolucion_pantalla||', p_activa: '||p_activa;
   
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_id_aplicacion IS NULL) THEN 
     campo_nulo:='id_aplicacion';
     RAISE E_NULO;
   ELSIF (p_version IS NULL) THEN 
     campo_nulo:='version';
     RAISE E_NULO;
   ELSIF (p_fecha_subida IS NULL) THEN 
     campo_nulo:='fecha_subida';
     RAISE E_NULO; 
   ELSIF (p_resolucion_pantalla IS NULL) THEN 
     campo_nulo:='resolucion_pantalla';
     RAISE E_NULO;
   ELSIF (p_activa IS NULL) THEN 
     campo_nulo:='activa';
     RAISE E_NULO;
   END IF;  

  --verificamos si existe la aplicacin
   SELECT COUNT (*)
   INTO existe
   FROM APLICACION
   WHERE id_aplicacion=p_id_aplicacion;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

   -- modificamos los datos de la aplicacin
   UPDATE APLICACION 
   SET version=p_version,
       fecha_subida=p_fecha_subida,
       url_demostracion=p_url_demostracion,
       resolucion_pantalla=p_resolucion_pantalla,
       activa=p_activa
   WHERE id_aplicacion=p_id_aplicacion;
     
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_APLICACION', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_APLICACION', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe una aplicacin con el mismo id_aplicacion             
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: la aplicacin no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_APLICACION', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo             
   WHEN OTHERS THEN          
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_APLICACION', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
              
END PR_MODIFICACION_APLICACION;