create or replace PROCEDURE PR_MODIFICACION_PRECIO (
   p_aplicacion IN PRECIO.aplicacion%TYPE,
   p_pais IN PRECIO.pais%TYPE,
   p_precio IN PRECIO.precio%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_MODIFICACION_PRECIO

   Propsito:           Modifica los datos de una entrada de la tabla PRECIO
                   
   Parmetros entrada:  p_aplicacion, p_pais, p_precio

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
existe_fk1 INTEGER;
existe_fk2 INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
E_FK EXCEPTION;
E_FK1 EXCEPTION;
E_FK2 EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:='p_aplicacion: '||p_aplicacion||', p_pais: '||p_pais||', p_precio: '||to_char(p_precio);
 
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   ELSIF (p_pais IS NULL) THEN 
     campo_nulo:='pais';
     RAISE E_NULO;
   ELSIF (p_precio IS NULL) THEN 
     campo_nulo:='precio';
     RAISE E_NULO; 
   END IF;

  --verificamos si existen la aplicacin y el pas
   SELECT COUNT (*)
   INTO existe_fk1
   FROM APLICACION
   WHERE id_aplicacion=p_aplicacion;

   SELECT COUNT (*)
   INTO existe_fk2
   FROM PAIS
   WHERE id_pais=p_pais;

   IF (existe_fk1=0 AND existe_fk2=0) THEN
     RAISE E_FK;
   ELSIF (existe_fk1=0) THEN 
     RAISE E_FK1;
   ELSIF (existe_fk2=0) THEN 
     RAISE E_FK2;
   END IF; 

  --verificamos si existe el precio
   SELECT COUNT (*)
   INTO existe
   FROM PRECIO
   WHERE aplicacion=p_aplicacion AND pais=p_pais;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

   -- modificamos los datos del precio
   UPDATE PRECIO 
   SET precio=p_precio
   WHERE aplicacion=p_aplicacion AND pais=p_pais;
     
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_PRECIO', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_PRECIO', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
  -- no finaliza con xito porque no existen la aplicacin ni el pas indicados     
   WHEN E_FK THEN
        RSP:='ERROR: la aplicacion y el pais no existen';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_PRECIO', param_entrada, 'RSP: '||RSP); 
        ROLLBACK;       
  -- no finaliza con xito porque no existe la aplicacin indicada     
   WHEN E_FK1 THEN
        RSP:='ERROR: la aplicacion no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_PRECIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe el pas indicado     
   WHEN E_FK2 THEN
        RSP:='ERROR: el pais no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_PRECIO', param_entrada, 'RSP: '||RSP);   
        ROLLBACK; 
  -- no finaliza con xito porque no existe un precio para esta aplicacin y pas     
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: el precio no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_PRECIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK; 
  -- no finaliza con xito por algn otro motivo    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_PRECIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK; 
              
END PR_MODIFICACION_PRECIO;