create or replace PROCEDURE PR_BAJA_DESCRIPCION (
   p_idioma IN DESCRIPCION.idioma%TYPE,
   p_aplicacion IN DESCRIPCION.aplicacion%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_BAJA_DESCRIPCION

   Propsito:           Elimina una entrada de la tabla DESCRIPCION
                   
   Parmetros entrada:  p_idioma, p_aplicacion

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs
   param_entrada:='p_idioma: '||p_idioma||', p_aplicacion: '||p_aplicacion;
                 
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_idioma IS NULL) THEN 
     campo_nulo:='idioma';
     RAISE E_NULO;
   ELSIF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   END IF; 

  --verificamos si existe la descripcin
   SELECT COUNT (*)
   INTO existe
   FROM DESCRIPCION
   WHERE idioma=p_idioma AND aplicacion=p_aplicacion;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

  -- eliminamos la descripcin
   DELETE FROM DESCRIPCION
   WHERE idioma=p_idioma AND aplicacion=p_aplicacion;

  -- finaliza con xito    
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESCRIPCION', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESCRIPCION', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe una descripcin en este idioma para esta aplicacin    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe una descripcin en este idioma para esta aplicacin';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESCRIPCION', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo     
   WHEN OTHERS THEN
        err_msg :=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESCRIPCION', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
              
END PR_BAJA_DESCRIPCION;