create or replace PROCEDURE PR_MODIFICACION_DESCRIPCION (
   p_idioma IN DESCRIPCION.idioma%TYPE,
   p_aplicacion IN DESCRIPCION.aplicacion%TYPE,
   p_descripcion IN DESCRIPCION.descripcion%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_MODIFICACION_DESCRIPCION

   Propsito:           Modifica los datos de una entrada de la tabla DESCRIPCION
                   
   Parmetros entrada:  p_idioma, p_aplicacion, p_descripcion

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
existe_fk INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
E_FK EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:='p_idioma: '||p_idioma||', p_aplicacion: '||p_aplicacion||', p_descripcion: '||p_descripcion;
   
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_idioma IS NULL) THEN 
     campo_nulo:='idioma';
     RAISE E_NULO;
   ELSIF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   ELSIF (p_descripcion IS NULL) THEN 
     campo_nulo:='descripcion';
     RAISE E_NULO; 
   END IF; 

  --verificamos si existe la aplicacin
   SELECT COUNT (*)
   INTO existe_fk
   FROM APLICACION
   WHERE id_aplicacion=p_aplicacion;

   IF (existe_fk=0) THEN
     RAISE E_FK;
   END IF;

  --verificamos si existe la descripcin
   SELECT COUNT (*)
   INTO existe
   FROM DESCRIPCION
   WHERE idioma=p_idioma AND aplicacion=p_aplicacion;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

   -- modificamos los datos de la descripcin
   UPDATE DESCRIPCION 
   SET descripcion=p_descripcion
   WHERE idioma=p_idioma AND aplicacion=p_aplicacion;
     
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DESCRIPCION', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DESCRIPCION', param_entrada, 'RSP: '||RSP);
        ROLLBACK; 
  -- no finaliza con xito porque no existe la aplicacin indicada   
   WHEN E_FK THEN
        RSP:='ERROR: no existe la aplicacin indicada';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESCRIPCION', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
  -- no finaliza con xito porque no existe una descripcin en este idioma para esta aplicacin    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe una descripcin en este idioma para esta aplicacin';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DESCRIPCION', param_entrada, 'RSP: '||RSP);
        ROLLBACK; 
  -- no finaliza con xito por algn otro motivo             
   WHEN OTHERS THEN          
        err_msg :=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DESCRIPCION', param_entrada, 'RSP: '||RSP);
        ROLLBACK; 
              
END PR_MODIFICACION_DESCRIPCION;