create or replace PROCEDURE PR_BAJA_DISPONIBLE (
   p_aplicacion IN DISPONIBLE.aplicacion%TYPE,
   p_sistema_operativo IN DISPONIBLE.sistema_operativo%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_BAJA_DISPONIBLE

   Propsito:           Elimina una entrada de la tabla DISPONIBLE
                   
   Parmetros entrada:  p_aplicacion, p_sistema_operativo

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs
   param_entrada:='p_aplicacion: '||p_aplicacion||', p_sistema_operativo: '||p_sistema_operativo;
         
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   ELSIF (p_sistema_operativo IS NULL) THEN 
     campo_nulo:='sistema_operativo';
     RAISE E_NULO;
   END IF; 

  --verificamos si existe la entrada en la tabla
   SELECT COUNT (*)
   INTO existe
   FROM DISPONIBLE
   WHERE aplicacion=p_aplicacion AND sistema_operativo=p_sistema_operativo;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

  -- eliminamos la entrada de la tabla
   DELETE FROM DISPONIBLE
   WHERE aplicacion=p_aplicacion AND sistema_operativo=p_sistema_operativo;

  -- finaliza con xito  
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DISPONIBLE', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DISPONIBLE', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe la entrada que se quiere eliminar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere eliminar';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DISPONIBLE', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DISPONIBLE', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_BAJA_DISPONIBLE;