create or replace PROCEDURE PR_ALTA_USUARIO (
   p_num_movil IN USUARIO.num_movil%TYPE,
   p_operador IN USUARIO.operador%TYPE,
   p_correo_electronico IN USUARIO.correo_electronico%TYPE,
   p_pais IN USUARIO.pais%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_ALTA_USUARIO

   Propsito:           Aade un nuevo usuario al sistema y actualiza la tabla 
			EST3_MEDIA_DESCARGAS
                   
   Parmetros entrada:  p_num_movil, p_operador, p_correo_electronico, p_pais

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
existe_fk INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_FK EXCEPTION;
E_YA_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:='p_num_movil: '||to_char(p_num_movil)||', p_operador: '||p_operador||', p_correo_electronico: '||p_correo_electronico||', p_pais: '||p_pais;   
   
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_num_movil IS NULL) THEN 
     campo_nulo:='num_movil';
     RAISE E_NULO;
   ELSIF (p_operador IS NULL) THEN 
     campo_nulo:='operador';
     RAISE E_NULO;
   ELSIF (p_correo_electronico IS NULL) THEN 
     campo_nulo:='correo_electronico';
     RAISE E_NULO; 
   ELSIF (p_pais IS NULL) THEN 
     campo_nulo:='pais';
     RAISE E_NULO;
   END IF; 

  --verificamos si existe el pas
   SELECT COUNT (*)
   INTO existe_fk
   FROM PAIS
   WHERE id_pais=p_pais;

   IF (existe_fk=0) THEN
     RAISE E_FK;
   END IF;

  --verificamos si existe el usuario
   SELECT COUNT (*)
   INTO existe
   FROM USUARIO
   WHERE num_movil=p_num_movil;

   IF (existe > 0) THEN
     RAISE E_YA_EXISTE;
   END IF;
   
  -- insertamos la nueva entrada en la tabla usuario
   INSERT INTO USUARIO (num_movil, operador, correo_electronico, pais)
   VALUES (p_num_movil, p_operador, p_correo_electronico, p_pais);

--------------------------Actualizamos EST3-------------------------
-- Lo actualizamos para todas las entradas de la tabla, ya que con el 
--planteamiento que hemos realizado de EST3 la cantidad de usuarios 
--del sistema afectar a la media de cualquier ao
   FOR rec IN (SELECT anyo_est3 FROM EST3_MEDIA_DESCARGAS) loop
      PR_EST3(rec.anyo_est3);
   END LOOP;   
-------------------------------------------------------------------- 
   
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_USUARIO', param_entrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe el pas indicado   
   WHEN E_FK THEN
        RSP:='ERROR: el pais no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_USUARIO', param_entrada, 'RSP: '||RSP); 
        ROLLBACK;  
  -- no finaliza con xito porque el usuario ya existe    
   WHEN E_YA_EXISTE THEN
        RSP:='ERROR: el usuario ya existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_USUARIO', param_entrada, 'RSP: '||RSP);
  -- no finaliza con xito por algn otro motivo      
   WHEN OTHERS THEN
        err_msg :=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
      
END PR_ALTA_USUARIO;