create or replace PROCEDURE PR_BAJA_USUARIO (
   p_num_movil IN USUARIO.num_movil%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_BAJA_USUARIO

   Propsito:           Elimina un usuario del sistema. Tambin elimina el posible 
			desarrollador y los dispositivos asociados al usuario y 
			actualiza la tabla EST3_MEDIA_DESCARGAS.
                   
   Parmetros entrada:  p_num_movil

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs
   param_entrada:='p_num_movil: '||to_char(p_num_movil);
         
  -- verificamos que el campo obligatorio no sea nulo
   IF (p_num_movil IS NULL) THEN 
     RAISE E_NULO;
   END IF; 

  --verificamos si existe el usuario
   SELECT COUNT (*)
   INTO existe
   FROM USUARIO
   WHERE num_movil=p_num_movil;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;
   
  -- eliminamos el desarrollador asociado con el usuario
   DELETE FROM DESARROLLADOR
   WHERE d_num_movil=p_num_movil;

  -- eliminamos los dispositivos asociados con el usuario
   DELETE FROM DISPOSITIVO
   WHERE usuario=p_num_movil;

  -- eliminamos el usuario
   DELETE FROM USUARIO
   WHERE num_movil=p_num_movil;

--------------------------Actualizamos EST3-------------------------
-- Lo actualizamos para todas las entradas de la tabla, ya que con el 
--planteamiento que hemos realizado de EST3 la cantidad de usuarios 
--del sistema afectar a la media de cualquier ao
   FOR rec IN (SELECT anyo_est3 FROM EST3_MEDIA_DESCARGAS) loop
      PR_EST3(rec.anyo_est3);
   END LOOP;   
-------------------------------------------------------------------- 

  -- finaliza con xito  
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_USUARIO', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque el campo obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo num_movil no puede ser nulo';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe la entrada que se quiere eliminar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere eliminar';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_BAJA_USUARIO;