create or replace PROCEDURE PR_MODIFICACION_USUARIO (
   p_num_movil IN USUARIO.num_movil%TYPE,
   p_operador IN USUARIO.operador%TYPE,
   p_correo_electronico IN USUARIO.correo_electronico%TYPE,
   p_pais IN USUARIO.pais%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_MODIFICACION_USUARIO

   Propsito:           Modifica una entrada en la tabla USUARIO
                   
   Parmetros entrada:  p_num_movil, p_operador, p_correo_electronico, p_pais

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
existe_fk INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_FK EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:='p_num_movil: '||to_char(p_num_movil)||', p_operador: '||p_operador||', p_correo_electronico: '||p_correo_electronico||', p_pais: '||p_pais;
   
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_num_movil IS NULL) THEN 
     campo_nulo:='num_movil';
     RAISE E_NULO;
   ELSIF (p_operador IS NULL) THEN 
     campo_nulo:='operador';
     RAISE E_NULO;
   ELSIF (p_correo_electronico IS NULL) THEN 
     campo_nulo:='correo_electronico';
     RAISE E_NULO; 
   ELSIF (p_pais IS NULL) THEN 
     campo_nulo:='pais';
     RAISE E_NULO;
   END IF; 

  --verificamos si existe el pas
   SELECT COUNT (*)
   INTO existe_fk
   FROM PAIS
   WHERE id_pais=p_pais;

   IF (existe_fk=0) THEN
     RAISE E_FK;
   END IF;

  --verificamos si existe el usuario
   SELECT COUNT (*)
   INTO existe
   FROM USUARIO
   WHERE num_movil=p_num_movil;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;
     
   -- modificamos los datos del usuario
   UPDATE USUARIO
   SET operador=p_operador,
       correo_electronico=p_correo_electronico,
       pais=p_pais
   WHERE num_movil=p_num_movil;   

  -- En este caso no hay que actualizar EST3 ya que el nmero de usuarios del sistema no ha cambiado 
        
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_USUARIO', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe el pas indicado   
   WHEN E_FK THEN
        RSP:='ERROR: el pais no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_USUARIO', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
  -- no finaliza con xito porque no existe la entrada que se quiere modificar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere modificar';  
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo        
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_MODIFICACION_USUARIO;