create or replace PROCEDURE PR_ALTA_EMPRESA (
   p_id_empresa IN EMPRESA.id_empresa%TYPE,
   p_nombre_empresa IN EMPRESA.nombre_empresa%TYPE,
   p_nombre_representante IN EMPRESA.nombre_representante%TYPE,
   p_pais IN EMPRESA.pais%TYPE,
   p_direccion_central IN EMPRESA.direccion_central%TYPE,
   p_telefono IN EMPRESA.telefono%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_ALTA_EMPRESA

   Propsito:           Aade una nueva empresa al sistema
                   
   Parmetros entrada:  p_id_empresa, p_nombre_empresa, p_nombre_representante, p_pais, 
			p_direccion_central, p_telefono

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
existe_fk INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_FK EXCEPTION;
E_YA_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:='p_id_empresa: '||to_char(p_id_empresa)||', p_nombre_empresa: '||p_nombre_empresa||', p_nombre_representante: '||p_nombre_representante||', p_pais: '||p_pais||', p_direccion_central: '||p_direccion_central||', p_telefono: '||to_char(p_telefono);   
   
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_id_empresa IS NULL) THEN 
     campo_nulo:='id_empresa';
     RAISE E_NULO;
   ELSIF (p_nombre_empresa IS NULL) THEN 
     campo_nulo:='nombre_empresa';
     RAISE E_NULO;
   ELSIF (p_nombre_representante IS NULL) THEN 
     campo_nulo:='nombre_representante';
     RAISE E_NULO; 
   ELSIF (p_pais IS NULL) THEN 
     campo_nulo:='pais';
     RAISE E_NULO;
   ELSIF (p_direccion_central IS NULL) THEN 
     campo_nulo:='direccion_central';
     RAISE E_NULO;
   ELSIF (p_telefono IS NULL) THEN 
     campo_nulo:='telefono';
     RAISE E_NULO;
   END IF; 

  --verificamos si existe el pas
   SELECT COUNT (*)
   INTO existe_fk
   FROM PAIS
   WHERE id_pais=p_pais;

   IF (existe_fk=0) THEN
     RAISE E_FK;
   END IF;

  --verificamos si existe la empresa
   SELECT COUNT (*)
   INTO existe
   FROM EMPRESA
   WHERE id_empresa=p_id_empresa;

   IF (existe > 0) THEN
     RAISE E_YA_EXISTE;
   END IF;
   
  -- insertamos la nueva entrada en la tabla EMPRESA
   INSERT INTO EMPRESA (id_empresa, nombre_empresa, nombre_representante, pais, direccion_central, telefono)
   VALUES (p_id_empresa, p_nombre_empresa, p_nombre_representante, p_pais, p_direccion_central, p_telefono);
   
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_EMPRESA', param_entrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_EMPRESA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe el pas indicado   
   WHEN E_FK THEN
        RSP:='ERROR: el pais no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_EMPRESA', param_entrada, 'RSP: '||RSP);  
        ROLLBACK; 
  -- no finaliza con xito porque la empresa ya existe    
   WHEN E_YA_EXISTE THEN
        RSP:='ERROR: la empresa ya existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_EMPRESA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo      
   WHEN OTHERS THEN         
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_EMPRESA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
      
END PR_ALTA_EMPRESA;