create or replace PROCEDURE PR_BAJA_EMPRESA (
   p_id_empresa IN EMPRESA.id_empresa%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_BAJA_EMPRESA

   Propsito:           Elimina una empresa del sistema. Tambin pone valor nulo en 
			el campo empresa de todos los desarrolladores asociados a dicha
			empresa.
                   
   Parmetros entrada:  p_id_empresa

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs
   param_entrada:='p_id_empresa: '||to_char(p_id_empresa);
         
  -- verificamos que el campo obligatorio no sea nulo
   IF (p_id_empresa IS NULL) THEN 
     RAISE E_NULO;
   END IF; 

  --verificamos si existe la empresa
   SELECT COUNT (*)
   INTO existe
   FROM EMPRESA
   WHERE id_empresa=p_id_empresa;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;
   
   -- modificamos los datos de los desarrolladores asociados a la empresa
   UPDATE DESARROLLADOR
   SET empresa=NULL
   WHERE empresa=p_id_empresa; 

  -- eliminamos la empresa de la tabla
   DELETE FROM EMPRESA
   WHERE id_empresa=p_id_empresa;

  -- finaliza con xito  
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_EMPRESA', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque el campo obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo id_empresa no puede ser nulo';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_EMPRESA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe la entrada que se quiere eliminar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere eliminar';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_EMPRESA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_EMPRESA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_BAJA_EMPRESA;