create or replace PROCEDURE PR_ALTA_DESARROLLADOR (
   p_num_movil IN DESARROLLADOR.d_num_movil%TYPE,
   p_empresa IN DESARROLLADOR.empresa%TYPE,
   p_operador IN USUARIO.operador%TYPE,
   p_correo_electronico IN USUARIO.correo_electronico%TYPE,
   p_pais IN USUARIO.pais%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_ALTA_DESARROLLADOR

   Propsito:           Aade un nuevo desarrollador al sistema. Para ello comprobamos
			si ya est registrado como usuario, y si no lo est lo aadimos
			tambin en la tabla USUARIO.
                   
   Parmetros entrada:  p_num_movil, p_empresa, p_operador, p_correo_electronico, p_pais

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe_u INTEGER;
existe INTEGER;
existe_fk INTEGER;
err_msg VARCHAR2(200);
RESP VARCHAR2(500);
E_NULO EXCEPTION;
E_USUARIO EXCEPTION;
E_FK EXCEPTION;
E_YA_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:='p_num_movil: '||to_char(p_num_movil)||', p_empresa: '||to_char(p_empresa);
   
  -- verificamos que el campo obligatorio no sea nulo
   IF (p_num_movil IS NULL) THEN 
     RAISE E_NULO;
   END IF; 

  --verificamos si existe como usuario
   SELECT COUNT (*)
   INTO existe_u
   FROM USUARIO
   WHERE num_movil=p_num_movil;
   -- si no existe como usuario, lo registramos como tal
   IF (existe_u=0) THEN
     PR_ALTA_USUARIO(p_num_movil, p_operador, p_correo_electronico, p_pais, RESP);
     IF (RESP <> 'OK') THEN
       RAISE E_USUARIO;
     END IF;
   END IF;

  --verificamos si existe la empresa
   IF (p_empresa IS NOT NULL) THEN
     SELECT COUNT (*)
     INTO existe_fk
     FROM EMPRESA
     WHERE id_empresa=p_empresa;

     IF (existe_fk=0) THEN
       RAISE E_FK;
     END IF;
   END IF;

  --verificamos si existe el desarrollador
   SELECT COUNT (*)
   INTO existe
   FROM DESARROLLADOR
   WHERE d_num_movil=p_num_movil;

   IF (existe > 0) THEN
     RAISE E_YA_EXISTE;
   END IF;
   
  -- insertamos la nueva entrada en la tabla DESARROLLADOR
   INSERT INTO DESARROLLADOR (d_num_movil, empresa)
   VALUES (p_num_movil, p_empresa);
   
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESARROLLADOR', param_entrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
  -- no finaliza con xito porque el valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo num_movil no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESARROLLADOR', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no se puede registrar como usuario   
   WHEN E_USUARIO THEN
        RSP:='Error al registrar el usuario. ERROR: '||RESP;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESARROLLADOR', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
  -- no finaliza con xito porque no existe la empresa   
   WHEN E_FK THEN
        RSP:='ERROR: la empresa no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESARROLLADOR', param_entrada, 'RSP: '||RSP);  
        ROLLBACK;  
  -- no finaliza con xito porque el desarrollador ya existe    
   WHEN E_YA_EXISTE THEN
        RSP:='ERROR: el desarrollador ya existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_EMPRESA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo   
   WHEN OTHERS THEN          
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESARROLLADOR', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
      
END PR_ALTA_DESARROLLADOR;