create or replace PROCEDURE PR_ALTA_DESARROLLA (
   p_desarrollador IN DESARROLLA.desarrollador%TYPE,
   p_aplicacion IN DESARROLLA.aplicacion%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_ALTA_DESARROLLA

   Propsito:           Aade una nueva entrada a la tabla DESARROLLA
                   
   Parmetros entrada:  p_desarrollador, p_aplicacion

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
existe_fk1 INTEGER;
existe_fk2 INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_FK EXCEPTION;
E_FK1 EXCEPTION;
E_FK2 EXCEPTION;
E_YA_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:='p_desarrollador: '||to_char(p_desarrollador)||', p_aplicacion: '||p_aplicacion;   
   
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_desarrollador IS NULL) THEN 
     campo_nulo:='desarrollador';
     RAISE E_NULO;
   ELSIF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   END IF; 

  --verificamos si existen el desarrollador y la aplicacin
   SELECT COUNT (*)
   INTO existe_fk1
   FROM DESARROLLADOR
   WHERE d_num_movil=p_desarrollador;

   SELECT COUNT (*)
   INTO existe_fk2
   FROM APLICACION
   WHERE id_aplicacion=p_aplicacion;

   IF (existe_fk1=0 AND existe_fk2=0) THEN
     RAISE E_FK;
   ELSIF (existe_fk1=0) THEN 
     RAISE E_FK1;
   ELSIF (existe_fk2=0) THEN 
     RAISE E_FK2;
   END IF;

  --verificamos si existe la entrada en la tabla desarrolla
   SELECT COUNT (*)
   INTO existe
   FROM DESARROLLA
   WHERE desarrollador=p_desarrollador AND aplicacion=p_aplicacion;

   IF (existe > 0) THEN
     RAISE E_YA_EXISTE;
   END IF;
   
  -- insertamos la nueva entrada en la tabla DESARROLLA
   INSERT INTO DESARROLLA (desarrollador, aplicacion)
   VALUES (p_desarrollador, p_aplicacion);
   
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESARROLLA', param_entrada, 'RSP: '||RSP);
   COMMIT;
   
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESARROLLA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existen el desarrollador ni la aplicacin indicados     
   WHEN E_FK THEN
        RSP:='ERROR: el desarrollador y la aplicacion no existen';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DISPONIBLE', param_entrada, 'RSP: '||RSP);
        ROLLBACK;        
  -- no finaliza con xito porque no existe el desarrollador indicado   
   WHEN E_FK1 THEN
        RSP:='ERROR: el desarrollador no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DISPONIBLE', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe la aplicacion indicada     
   WHEN E_FK2 THEN
        RSP:='ERROR: la aplicacion no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DISPONIBLE', param_entrada, 'RSP: '||RSP);  
        ROLLBACK;  
  -- no finaliza con xito porque la entrada ya existe en la tabla  
   WHEN E_YA_EXISTE THEN
        RSP:='ERROR: la entrada ya existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_EMPRESA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo     
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESARROLLA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
      
END PR_ALTA_DESARROLLA;