create or replace PROCEDURE PR_BAJA_DESARROLLA (
   p_desarrollador IN DESARROLLA.desarrollador%TYPE,
   p_aplicacion IN DESARROLLA.aplicacion%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_BAJA_DESARROLLA

   Propsito:           Elimina una entrada de la tabla DESARROLLA
                   
   Parmetros entrada:  p_desarrollador, p_aplicacion

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs
   param_entrada:='p_desarrollador: '||to_char(p_desarrollador)||', p_aplicacion: '||p_aplicacion;
         
  -- verificamos que el campo obligatorio no sea nulo
   IF (p_desarrollador IS NULL) THEN 
     campo_nulo:='desarrollador';
     RAISE E_NULO;
   ELSIF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   END IF; 

  --verificamos si existe la entrada en la tabla desarrolla
   SELECT COUNT (*)
   INTO existe
   FROM DESARROLLA
   WHERE desarrollador=p_desarrollador AND aplicacion=p_aplicacion;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

  -- eliminamos la entrada de la tabla
   DELETE FROM DESARROLLA
   WHERE desarrollador=p_desarrollador AND aplicacion=p_aplicacion;

  -- finaliza con xito  
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESARROLLA', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESARROLLA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe la entrada que se quiere eliminar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere eliminar';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESARROLLA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESARROLLA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_BAJA_DESARROLLA;