create or replace PROCEDURE PR_BAJA_DISPOSITIVO (
   p_codigo_imei IN DISPOSITIVO.codigo_imei%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_BAJA_DISPOSITIVO

   Propsito:           Elimina un dispositivo del sistema y las descargas
			asociadas a el
                   
   Parmetros entrada:  p_codigo_imei

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs
   param_entrada:='p_codigo_imei: '||to_char(p_codigo_imei);
         
  -- verificamos que el campo obligatorio no sea nulo
   IF (p_codigo_imei IS NULL) THEN 
     RAISE E_NULO;
   END IF; 

  --verificamos si existe el dispositivo
   SELECT COUNT (*)
   INTO existe
   FROM DISPOSITIVO
   WHERE codigo_imei=p_codigo_imei;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

  -- eliminamos las descargas asociadas
   DELETE FROM DESCARGA
   WHERE dispositivo=p_codigo_imei;

  -- eliminamos la entrada de la tabla
   DELETE FROM DISPOSITIVO
   WHERE codigo_imei=p_codigo_imei;

  -- finaliza con xito  
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque el campo obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo codigo_imei no puede ser nulo';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe la entrada que se quiere eliminar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere eliminar';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DISPOSITIVO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_BAJA_DISPOSITIVO;
