create or replace PROCEDURE PR_MODIFICACION_DISPOSITIVO (
   p_codigo_imei IN DISPOSITIVO.codigo_imei%TYPE,
   p_sistema_operativo IN DISPOSITIVO.sistema_operativo%TYPE,
   p_modelo IN DISPOSITIVO.modelo%TYPE,
   p_resolucion IN DISPOSITIVO.resolucion%TYPE,
   p_usuario IN DISPOSITIVO.usuario%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_MODIFICACION_DISPOSITIVO

   Propsito:           Modifica una entrada de la tabla DISPOSITIVO
                   
   Parmetros entrada:  p_codigo_imei, p_sistema_operativo, p_modelo, p_resolucion, 
			p_usuario

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
existe_fk INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
E_FK EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs    
   param_entrada:='p_codigo_imei: '||to_char(p_codigo_imei)||', p_sistema_operativo: '||p_sistema_operativo||', p_modelo: '||p_modelo||', p_resolucion: '||p_resolucion||', p_usuario: '||to_char(p_usuario);
   
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_codigo_imei IS NULL) THEN 
     campo_nulo:='codigo_imei';
     RAISE E_NULO;
   ELSIF (p_sistema_operativo IS NULL) THEN 
     campo_nulo:='sistema_operativo';
     RAISE E_NULO;
   ELSIF (p_modelo IS NULL) THEN 
     campo_nulo:='modelo';
     RAISE E_NULO; 
   ELSIF (p_resolucion IS NULL) THEN 
     campo_nulo:='resolucion';
     RAISE E_NULO;
   ELSIF (p_usuario IS NULL) THEN 
     campo_nulo:='usuario';
     RAISE E_NULO;
   END IF;

  --verificamos si existe el usuario
   SELECT COUNT (*)
   INTO existe_fk
   FROM USUARIO
   WHERE num_movil=p_usuario;

   IF (existe_fk=0) THEN
     RAISE E_FK;
   END IF; 

  --verificamos si existe el dispositivo
   SELECT COUNT (*)
   INTO existe
   FROM DISPOSITIVO
   WHERE codigo_imei=p_codigo_imei;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;
     
   -- modificamos los datos del dispositivo
   UPDATE DISPOSITIVO
   SET sistema_operativo=p_sistema_operativo,
       modelo=p_modelo,
       resolucion=p_resolucion,
       usuario=p_usuario
   WHERE codigo_imei=p_codigo_imei;     
        
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DISPOSITIVO', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DISPOSITIVO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe el usuario indicado   
   WHEN E_FK THEN
        RSP:='ERROR: el usuario no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DISPOSITIVO', param_entrada, 'RSP: '||RSP);  
        ROLLBACK;
  -- no finaliza con xito porque no existe la entrada que se quiere modificar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere modificar';  
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DISPOSITIVO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo        
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DISPOSITIVO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_MODIFICACION_DISPOSITIVO;