create or replace PROCEDURE PR_BAJA_DESCARGA (
   p_aplicacion IN DESCARGA.aplicacion%TYPE,
   p_dispositivo IN DESCARGA.dispositivo%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_BAJA_DESCARGA

   Propsito:           Elimina una descarga del sistema. Adems actualiza las 
			tablas del mdulo estadstico.
                   
   Parmetros entrada:  p_aplicacion, p_dispositivo

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
p_fecha DATE;
p_pais CHAR(2);
E_NULO EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs
   param_entrada:='p_aplicacion: '||p_aplicacion||', p_dispositivo: '||to_char(p_dispositivo);
         
  -- verificamos que el campo obligatorio no sea nulo
   IF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   ELSIF (p_dispositivo IS NULL) THEN 
     campo_nulo:='dispositivo';
     RAISE E_NULO;
   END IF; 

  --verificamos si existe la descarga
   SELECT COUNT (*)
   INTO existe
   FROM DESCARGA
   WHERE aplicacion=p_aplicacion AND dispositivo=p_dispositivo;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

  --damos a p_fecha y p_pais el valor de la fecha y el pais de la descarga
  --para utilizarlos para actualizar el mdulo estadstico
   SELECT fecha, pais
   INTO p_fecha, p_pais
   FROM DESCARGA
   WHERE aplicacion=p_aplicacion AND dispositivo=p_dispositivo;

  -- eliminamos la entrada de la tabla
   DELETE FROM DESCARGA
   WHERE aplicacion=p_aplicacion AND dispositivo=p_dispositivo;

--------------------------Actualizamos EST1-------------------------
   PR_EST1();
--------------------------------------------------------------------

--------------------------Actualizamos EST2-------------------------
   PR_EST2();
--------------------------------------------------------------------  

--------------------------Actualizamos EST3-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga

   PR_EST3(EXTRACT(YEAR FROM p_fecha));
--------------------------------------------------------------------

--------------------------Actualizamos EST4-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga

   PR_EST4(EXTRACT(YEAR FROM p_fecha));
-------------------------------------------------------------------- 

--------------------------Actualizamos EST5-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga

   PR_EST5(EXTRACT(YEAR FROM p_fecha));
-------------------------------------------------------------------- 

--------------------------Actualizamos EST6-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga
--y al pas donde se realiza la descarga

   PR_EST6(EXTRACT(YEAR FROM p_fecha), p_pais);
-------------------------------------------------------------------- 

--------------------------Actualizamos EST7-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga
--y al pas donde se realiza la descarga

   PR_EST7(EXTRACT(YEAR FROM p_fecha), p_pais);
-------------------------------------------------------------------- 

--------------------------Actualizamos EST8-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga
--y al pas donde se realiza la descarga

   PR_EST8(EXTRACT(YEAR FROM p_fecha), p_pais);
-------------------------------------------------------------------- 

  -- finaliza con xito  
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESCARGA', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo ' || campo_nulo || ' no puede ser nulo';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESCARGA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe la entrada que se quiere eliminar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere eliminar';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESCARGA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo    
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: ' || err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_BAJA_DESCARGA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_BAJA_DESCARGA;