create or replace PROCEDURE PR_MODIFICACION_DESCARGA (
   p_aplicacion IN DESCARGA.aplicacion%TYPE,
   p_dispositivo IN DESCARGA.dispositivo%TYPE,
   p_fecha IN DESCARGA.fecha%TYPE,
   p_forma_de_pago IN DESCARGA.forma_de_pago%TYPE,
   p_precio IN DESCARGA.usuario%TYPE,
   p_operador IN DESCARGA.usuario%TYPE,
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_MODIFICACION_DESCARGA

   Propsito:           Modifica una entrada de la tabla DESCARGA. Adems 
			actualiza las tablas del mdulo estadstico.
                   
   Parmetros entrada:  p_aplicacion, p_dispositivo, p_resolucion, 
			p_forma_de_pago, p_precio, p_operador

   Parmetros de salida: RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
existe INTEGER;
existe_fk1 INTEGER;
existe_fk2 INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
p_usuario NUMBER;
p_pais CHAR(2);
p_activa CHAR(2);
E_NULO EXCEPTION;
E_FK1 EXCEPTION;
E_FK2 EXCEPTION;
E_NO_ACTIVA EXCEPTION;
E_NO_EXISTE EXCEPTION;
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs    
   param_entrada:='p_aplicacion: '||p_aplicacion||', p_dispositivo: '||to_char(p_dispositivo)||', p_fecha: '||to_char(p_fecha)||', p_forma_de_pago: '||p_forma_de_pago||', p_precio: '||to_char(p_precio)||', p_operador: '||p_operador;
   
  -- verificamos que los campos obligatorios no sean nulos
   IF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   ELSIF (p_dispositivo IS NULL) THEN 
     campo_nulo:='dispositivo';
     RAISE E_NULO;
   ELSIF (p_fecha IS NULL) THEN 
     campo_nulo:='resolucion';
     RAISE E_NULO;
   ELSIF (p_forma_de_pago IS NULL) THEN 
     campo_nulo:='forma_de_pago';
     RAISE E_NULO;
   ELSIF (p_precio IS NULL) THEN 
     campo_nulo:='precio';
     RAISE E_NULO;
   ELSIF (p_operador IS NULL) THEN 
     campo_nulo:='operador';
     RAISE E_NULO;
   END IF; 

  --verificamos si existe la aplicacion
   SELECT COUNT (*)
   INTO existe_fk1
   FROM APLICACION
   WHERE id_aplicacion=p_aplicacion;

   IF (existe_fk1=0) THEN
     RAISE E_FK1;
   END IF;

  --verificamos si la aplicacion est disponible para descarga
   SELECT activa
   INTO p_activa
   FROM APLICACION
   WHERE id_aplicacion=p_aplicacion;

   IF (p_activa='NO') THEN
     RAISE E_NO_ACTIVA;
   END IF;

  --verificamos si existe el dispositivo
   SELECT COUNT (*)
   INTO existe_fk2
   FROM DISPOSITIVO
   WHERE codigo_imei=p_dispositivo;

   IF (existe_fk2=0) THEN
     RAISE E_FK2;
   END IF;

  --verificamos si existe la descarga
   SELECT COUNT (*)
   INTO existe
   FROM DESCARGA
   WHERE aplicacion=p_aplicacion AND dispositivo=p_dispositivo;

   IF (existe=0) THEN
     RAISE E_NO_EXISTE;
   END IF;

  --damos a p_usuario el valor del usuario del dispositivo
   SELECT usuario
   INTO p_usuario
   FROM DISPOSITIVO
   WHERE codigo_imei=p_dispositivo;

  --damos a p_pais el valor del pais del usuario
   SELECT pais
   INTO p_pais
   FROM USUARIO
   WHERE num_movil=p_usuario;
     
   -- modificamos los datos de la descarga
   UPDATE DESCARGA
   SET dispositivo=p_dispositivo,
       usuario=p_usuario,
       pais=p_pais,
       fecha=p_fecha,
       forma_de_pago=p_forma_de_pago,
       precio=p_precio,
       operador=p_operador
   WHERE aplicacion=p_aplicacion;   

--------------------------Actualizamos EST1-------------------------
   PR_EST1();
--------------------------------------------------------------------  

--------------------------Actualizamos EST2-------------------------
   PR_EST2();
--------------------------------------------------------------------  

--------------------------Actualizamos EST3-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga

   PR_EST3(EXTRACT(YEAR FROM p_fecha));
-------------------------------------------------------------------- 

--------------------------Actualizamos EST4-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga

   PR_EST4(EXTRACT(YEAR FROM p_fecha));
-------------------------------------------------------------------- 

--------------------------Actualizamos EST5-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga

   PR_EST5(EXTRACT(YEAR FROM p_fecha));
-------------------------------------------------------------------- 

--------------------------Actualizamos EST6-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga
--y al pas donde se realiza la descarga

   PR_EST6(EXTRACT(YEAR FROM p_fecha), p_pais);
-------------------------------------------------------------------- 

--------------------------Actualizamos EST7-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga
--y al pas donde se realiza la descarga

   PR_EST7(EXTRACT(YEAR FROM p_fecha), p_pais);
-------------------------------------------------------------------- 

--------------------------Actualizamos EST8-------------------------
-- Lo actualizamos slo para el valor de la tabla que cambiar que ser 
--el correspondiente al ao de la fecha de realizacin de la descarga
--y al pas donde se realiza la descarga

   PR_EST8(EXTRACT(YEAR FROM p_fecha), p_pais);
-------------------------------------------------------------------- 
        
  -- finaliza con xito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DESCARGA', param_entrada, 'RSP: '||RSP);
   
   COMMIT;
EXCEPTION
  -- no finaliza con xito porque algn valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DESCARGA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe la aplicacion indicada   
   WHEN E_FK1 THEN
        RSP:='ERROR: la aplicacion no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESCARGA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque no existe el dispositivo indicado   
   WHEN E_FK2 THEN
        RSP:='ERROR: el dispositivo no existe';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESCARGA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito porque la aplicacion indicada no est disponible para descarga  
   WHEN E_NO_ACTIVA THEN
        RSP:='ERROR: la aplicacion no se encuentra disponible para descarga';
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_ALTA_DESCARGA', param_entrada, 'RSP: '||RSP);   
        ROLLBACK; 
  -- no finaliza con xito porque no existe la entrada que se quiere modificar    
   WHEN E_NO_EXISTE THEN
        RSP:='ERROR: no existe la entrada que se quiere modificar';  
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DESCARGA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo        
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_MODIFICACION_DESCARGA', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
        
END PR_MODIFICACION_DESCARGA;