CREATE OR REPLACE PROCEDURE PR_CONS_DESARROLLADORES_PAIS (
   p_pais IN PAIS.id_pais%TYPE,
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_CONS_DESARROLLADORES_PAIS

   Propsito:           Dado un pas, obtenemos los desarrolladores de dicho pas
			con todos sus datos, incluyendo el nmero de aplicaciones 
			diferentes publicadas
                   
   Parmetros entrada:  p_pais  

   Parmetros de salida: REFCURSOR, RSP          

   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
num INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
    
BEGIN
  -- creamos el parmetro que utilizaremos como parmetro de entrada para los logs   
   param_entrada:= ', p_pais: '||p_pais;

  -- verificamos que el campo obligatorio no sea nulo
   IF (p_pais IS NULL) THEN 
     campo_nulo:= 'pais';
     RAISE E_NULO;
   END IF;


  OPEN REFCURSOR FOR
  SELECT d.d_num_movil, d.empresa, u.operador, u.correo_electronico, u.pais, COUNT(des.aplicacion) AS num
  FROM (DESARROLLADOR d JOIN USUARIO u ON d.d_num_movil=u.num_movil) 
  JOIN DESARROLLA des ON d.d_num_movil=des.desarrollador
  WHERE u.pais=p_pais;
  
  -- finaliza con xito
   RSP:= 'OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESARROLLADORES_PAIS', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finaliza con xito porque el valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo pais no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESARROLLADORES_PAIS', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con xito por algn otro motivo         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESARROLLADORES_PAIS', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
   
END PR_CONS_DESARROLLADORES_PAIS;