CREATE OR REPLACE PROCEDURE PR_CONS_APLICACIONES_ACTIVAS (
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_CONS_APLICACIONES_ACTIVAS

   Propósito:           Obtendremos un listado con todas las aplicaciones activas
			y sus datos principales, ordenadas por el número de descargas
			que han tenido hasta el momento a nivel mundial
                   
   Parámetros entrada:  sin parámetros 

   Parámetros de salida: REFCURSOR, RSP          

   Retorna:             'OK' si la ejecución ha finalizado con éxito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
err_msg VARCHAR2(200);
num INTEGER;
    
BEGIN
  -- creamos el parámetro que utilizaremos como parámetro de entrada para los logs   
   param_entrada:='sin parámetros';

  OPEN REFCURSOR FOR
  SELECT a.id_aplicacion, a.version, a.fecha_subida, a.resolucion_pantalla, COUNT(d.fecha) AS num
  FROM APLICACION a JOIN DESCARGA d ON a.id_aplicacion=d.aplicacion
  ORDER BY num DESC;
  
  -- finaliza con éxito
   RSP:= 'OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_APLICACIONES_ACTIVAS', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finaliza con éxito        
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_APLICACIONES_ACTIVAS', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
   
END PR_CONS_APLICACIONES_ACTIVAS;