CREATE OR REPLACE PROCEDURE PR_CONS_DESCARGAS_APLICACION (
   p_aplicacion IN APLICACION.id_aplicacion%TYPE,
   p_anyo NUMBER,
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_CONS_DESCARGAS_APLICACION 

   Propósito:           Dados una aplicación y un año concretos, obtendremos un
			listado con los países en que la aplicación se ha 
			descargado ese año y el número de descargas que ha tenido 
			en cada país
                   
   Parámetros entrada:  p_aplicacion, p_anyo  

   Parámetros de salida: REFCURSOR, RSP          

   Retorna:             'OK' si la ejecución ha finalizado con éxito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
num INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
    
BEGIN
  -- creamos el parámetro que utilizaremos como parámetro de entrada para los logs   
   param_entrada:= 'p_aplicacion: '||p_aplicacion||', p_anyo: '||to_char(p_anyo);

  -- verificamos que los campos obligatorios no sean nulos
   IF (p_aplicacion IS NULL) THEN 
     campo_nulo:='aplicacion';
     RAISE E_NULO;
   ELSIF (p_anyo IS NULL) THEN 
     campo_nulo:='anyo';
     RAISE E_NULO;
   END IF;


  OPEN REFCURSOR FOR
  SELECT pais, COUNT(fecha) AS num
  FROM DESCARGA d
  WHERE aplicacion=p_aplicacion AND EXTRACT(YEAR FROM d.fecha)=p_anyo
  ORDER BY num DESC;
  
  -- finaliza con éxito
   RSP:= 'OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESCARGAS_APLICACION ', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finaliza con éxito porque algún valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:= 'ERROR: el campo '||campo_nulo||' no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESCARGAS_APLICACION ', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con éxito por algún otro motivo         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESCARGAS_APLICACION ', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
   
END PR_CONS_DESCARGAS_APLICACION ;