CREATE OR REPLACE PROCEDURE PR_CONS_DESCARGAS_USUARIO (
   p_usuario IN USUARIO.num_movil%TYPE,
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_CONS_DESCARGAS_USUARIO

   Propósito:           Dado un usuario obtendremos una lista de todas las descargas
			que ha realizado con los datos de cada descarga.
                   
   Parámetros entrada:  p_usuario  

   Parámetros de salida: REFCURSOR, RSP          

   Retorna:             'OK' si la ejecución ha finalizado con éxito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
    
BEGIN
  -- creamos el parámetro que utilizaremos como parámetro de entrada para los logs   
   param_entrada:=', p_usuario: '||to_char(p_usuario);

  -- verificamos que el campo obligatorio no sea nulo
   IF (p_usuario IS NULL) THEN 
     campo_nulo:='usuario';
     RAISE E_NULO;
   END IF;


  OPEN REFCURSOR FOR
  SELECT * 
  FROM DESCARGA
  WHERE usuario=p_usuario
  ORDER BY fecha DESC;
  
  -- finaliza con éxito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESCARGAS_USUARIO', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finaliza con éxito porque el valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo usuario no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESCARGAS_USUARIO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con éxito por algún otro motivo         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_DESCARGAS_USUARIO', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
   
END PR_CONS_DESCARGAS_USUARIO;