CREATE OR REPLACE PROCEDURE PR_CONS_USUARIOS_MAX_GASTO (
   p_anyo NUMBER,
   REFCURSOR OUT SYS_REFCURSOR, 
   RSP OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:              PR_CONS_USUARIOS_MAX_GASTO

   Propósito:           Dado un año concreto, obtendremos el listado de los 20 
			usuarios que más dinero han gastado en descargas, ordenados
			de mayor a menor gasto
                   
   Parámetros entrada:  p_anyo  

   Parámetros de salida: REFCURSOR, RSP          

   Retorna:             'OK' si la ejecución ha finalizado con éxito
                        'ERROR: tipo de error' si ha fracasado
******************************************************************************/
param_entrada VARCHAR2(500);
suma INTEGER;
err_msg VARCHAR2(200);
campo_nulo VARCHAR2(20);
E_NULO EXCEPTION;
    
BEGIN
  -- creamos el parámetro que utilizaremos como parámetro de entrada para los logs   
   param_entrada:=', p_anyo: '||to_char(p_anyo);

  -- verificamos que el campo obligatorio no sea nulo
   IF (p_anyo IS NULL) THEN 
     campo_nulo:='anyo';
     RAISE E_NULO;
   END IF;


  OPEN REFCURSOR FOR
  SELECT usuario, suma
  FROM (SELECT d.usuario AS usuario, SUM(d.precio) AS suma FROM DESCARGA d JOIN USUARIO u ON d.usuario=u.num_movil 
       WHERE EXTRACT(YEAR FROM d.fecha)=p_anyo ORDER BY suma DESC)
  WHERE rownum<=20;
  
  -- finaliza con éxito
   RSP:='OK';
   PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_USUARIOS_MAX_GASTO', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
EXCEPTION
  -- no finaliza con éxito porque el valor obligatorio es nulo
   WHEN E_NULO THEN
        RSP:='ERROR: el campo anyo no puede ser nulo'; 
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_USUARIOS_MAX_GASTO', param_entrada, 'RSP: '||RSP);
        ROLLBACK;
  -- no finaliza con éxito por algún otro motivo         
   WHEN OTHERS THEN
        err_msg:=SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR: '||err_msg;
        PR_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'PR_CONS_USUARIOS_MAX_GASTO', param_entrada, 'RSP: '||RSP); 
        ROLLBACK; 
   
END PR_CONS_USUARIOS_MAX_GASTO;